import logging

from rest_framework.authentication import BaseAuthentication
from django.conf import settings
from keycloak import KeycloakOpenID
from .models import OpenIdConnectProfile

logger = logging.getLogger(__file__)


if "API" in settings.KEYCLOAK_CLIENTS:
    API_CLIENT = KeycloakOpenID(
        settings.KEYCLOAK_CLIENTS["API"]["URL"],
        settings.KEYCLOAK_CLIENTS["API"]["REALM"],
        settings.KEYCLOAK_CLIENTS["API"]["CLIENT_ID"],
        settings.KEYCLOAK_CLIENTS["API"]["CLIENT_SECRET"],
    )
else:
    API_CLIENT = None


class KeycloakDRFAuthentication(BaseAuthentication):
    """Authentication backend for rest framework."""

    def authenticate(self, request):
        auth = request.META.get("HTTP_AUTHORIZATION")
        if not auth:
            return None
        auth_parts = auth.split()
        if not len(auth_parts) > 1:
            return None
        profile = OpenIdConnectProfile.from_token(auth_parts[1], client=API_CLIENT)
        if not profile:
            return None
        return (profile.user, profile)



from django.conf import settings
from django.contrib.auth.models import User
from keycloak import KeycloakAdmin, KeycloakOpenID


default_conf = settings.KEYCLOAK_CLIENTS["DEFAULT"]
DEFAULT_CLIENT = KeycloakOpenID(
    server_url=default_conf["URL"],
    realm_name=default_conf["REALM"],
    client_id=default_conf["CLIENT_ID"],
    client_secret_key=default_conf["CLIENT_SECRET"],
)


if "ADMIN" in settings.KEYCLOAK_CLIENTS:
    admin_conf = settings.KEYCLOAK_CLIENTS["ADMIN"]
    ADMIN_CLIENT = KeycloakAdmin(
        server_url=default_conf["URL"],
        username=admin_conf["USERNAME"],
        password=admin_conf["PASSWORD"],
        user_realm_name=admin_conf.get("REALM", "master"),
        realm_name=default_conf["REALM"],
    )
else:
    ADMIN_CLIENT = None


def create_keycloak_user(user: User, password: str | None = None):
    """Create a keycloak user from a Django user."""
    if not ADMIN_CLIENT:
        raise ValueError("Cannot create a user, the admin client is not configured")
    credentials = []
    if password:
        credentials.append({"type": "password", "value": password})
    ADMIN_CLIENT.create_user(
        {
            "email": user.email,
            "username": user.username,
            "enabled": user.is_active,
            "firstName": user.first_name,
            "lastName": user.last_name,
            "credentials": credentials,
        }
    )



import logging

from django.shortcuts import resolve_url
from django.conf import settings
from django.contrib.auth import authenticate, login, logout
from django.http.response import (
    HttpResponseBadRequest,
    HttpResponseServerError,
    HttpResponseRedirect
)
from django.urls.base import reverse
from keycloak.exceptions import KeycloakConnectionError
from django.views.generic.base import RedirectView

from .models import Nonce, DEFAULT_CLIENT


logger = logging.getLogger(__name__)


class Login(RedirectView):
    """Login View."""

    def has_admin_permission(self, request):
        """Check whether the user can access the admin site."""
        return request.user.is_active and request.user.is_staff

    def get_redirect_url(self, *args, **kwargs):
        """Redirects to the OIDC authorization URL."""
        # If the user is already authenticated there's no need to
        # refer them to keycloak
        if self.request.user.is_authenticated:
            next_path = self.request.GET.get("next")
            if self.has_admin_permission(self.request):
                return next_path or "/"
            # Want to prevent an infinite loop, if the user has been authenticated
            # but is not a staff user they will bounce back between the admin page
            # which redirects them to login because they can't view it, which
            # succeeds because they're logged into the session, which directs them back
            # to the admin page.
            # TODO: Should warn the user that they're about to be logged out
            return f'{reverse("keycloak_logout")}?next={next_path}'
        nonce = Nonce.objects.create(
            redirect_uri=self.request.build_absolute_uri(
                location=reverse("keycloak_login_complete")
            ),
            next_path=self.request.GET.get("next"),
        )
        self.request.session["oidc_state"] = str(nonce.state)
        try:
            authorization_url = DEFAULT_CLIENT.auth_url(
                redirect_uri=nonce.redirect_uri, state=str(nonce.state)
            )
        except KeycloakConnectionError as kce:
            logger.error("Unable to fetch auth URL, is keycloak running? (Error: %s)", kce)
            return None
        return authorization_url


class LoginComplete(RedirectView):
    """Login Complete View."""

    def get(self, *args, **kwargs):
        request = self.request

        if "error" in request.GET:
            return HttpResponseServerError(request.GET["error"])

        if "code" not in request.GET and "state" not in request.GET:
            return HttpResponseBadRequest()

        if (
            "oidc_state" not in request.session
            or request.GET["state"] != request.session["oidc_state"]
        ):
            # Missing or incorrect state; login again.
            return HttpResponseRedirect(reverse("keycloak_login"))

        nonce = Nonce.objects.get(state=request.GET["state"])
        user = authenticate(
            request=request, code=request.GET["code"], redirect_uri=nonce.redirect_uri
        )
        if user and user.is_authenticated:
            login(request, user)
        nonce.delete()
        return HttpResponseRedirect(nonce.next_path or "/")


class Logout(RedirectView):
    """Logout View."""

    def get_redirect_url(self, *args, **kwargs):
        profile = getattr(self.request.user, "oidc_profile", None)
        if profile:
            profile.client.logout(profile.refresh_token)
            profile.delete()

        logout(self.request)

        if settings.LOGOUT_REDIRECT_URL:
            return resolve_url(settings.LOGOUT_REDIRECT_URL)

        return reverse("keycloak_login")


login_view = Login.as_view()
login_complete_view = LoginComplete.as_view()
logout_view = Logout.as_view()
